/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ciphermemo;

import java.util.Arrays;
import java.util.zip.CRC32;

/**
 *
 * @author kanemitsu
 */
public class Cipher {

    private Sfmt mt;
    private CRC32 digest;
    private int hash;
    private int hashPosition = 0;
    private int randomBlockLength = 8;

    public static byte[] convertToByteArray(int data){
        byte[] result = new byte[4];
        convertToByteArray(data, result);
        return result;
    }

    public static void convertToByteArray(int data, byte[] dest){
        dest[0] = (byte)((data & 0xFF000000) >>> 24);
        dest[1] = (byte)((data & 0x00FF0000) >>> 16);
        dest[2] = (byte)((data & 0x0000FF00) >>> 8);
        dest[3] = (byte)(data & 0x000000FF);
    }

    public static int convertToInt(byte[] data) {
        return ((data[0] & 0xFF) << 24) | ((data[1] & 0xFF) << 16) | ((data[2] & 0xFF) << 8) | (data[3] & 0xFF);
    }
    
    public void dispose() {
        mt.InitMt(0);
        mt = null;
        digest = null;
        hashPosition = 0;
        hash = 0;
    }

    public Cipher(byte[] key) {
        /*
         * byte[] を int[] に変換
         */
        int len = key.length / 4;
        boolean flag = false;
        if (key.length % 4 > 0) {
            len++;
            flag = true;
        }
        
        int[] key_int = new int[len];
        if (flag) len--;
        
        int ib = 0;
        for (int i = 0; i < len; ++i) {
            //4バイトまとめてintにする
            key_int[i] = (key[ib] & 0xFF) | ((key[ib + 1] & 0xFF) << 8) | ((key[ib + 2] & 0xFF) << 16) | ((key[ib + 3] & 0xFF) << 24);
            ib += 4;
        }
        
        //4バイト未満の余りを処理する
        if (flag) {
            len = key.length % 4;
            int temp = (key[ib] & 0xFF);
            if (len > 1) temp |= (key[ib + 1] & 0xFF) << 8;
            if (len > 2) temp |= (key[ib + 2] & 0xFF) << 16;
            key_int[key_int.length - 1] = temp;
        }

        //Sfmtを生成
        this.mt = new Sfmt(key_int);
        
        //100万回捨てる
        /*
        for (int i = 0; i < 1000000; ++i) {
            this.mt.NextMt();
        }
         * 
         */
        
        //int[]に変換した鍵を削除
        Arrays.fill(key_int, 0);

        digest = new CRC32();
    }

    /**
     * 乱数を指定回数捨てる
     * @param count 捨てる回数
     */
    public void skip(int count) {
        for (int i = 0; i < count; ++i) {
            this.mt.NextMt();
        }
    }

    public void setRandomBlockLength(int randomBlockLength) {
        this.randomBlockLength = (randomBlockLength & 7) + 2;
    }
    
    
    public void crypt(byte[] data) {

        /*
         * 暗号化・復号処理
         */
        for (int i = 0; i < data.length; ++i) {

            //0なら乱数を更新
            if (hashPosition == 0) {
                //乱数1ブロックのバイト数を計算
                int temp = this.randomBlockLength * 4;
                                
                byte[] buf = new byte[temp];
                for (int i2 = 0; i2 < temp; i2 += 4) {
                    int r = this.mt.NextMt();
                    buf[i2 + 0] = (byte)(r & 0x000000FF);
                    buf[i2 + 1] = (byte)((r & 0x0000FF00) >>> 8);
                    buf[i2 + 2] = (byte)((r & 0x00FF0000) >>> 16);
                    buf[i2 + 3] = (byte)((r & 0xFF000000) >>> 24);
                }

                this.digest.update(buf);
                hash = (int)(digest.getValue() & 0xFFFFFFFF);
            }
            
            data[i] ^= (byte)(hash & 0xFF);
            hash >>>= 8;
            hashPosition = (hashPosition + 1) & 3;
        }
        
    }

}
