/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * NewJFrame.java
 *
 * Created on 2010/11/02, 20:33:30
 */

package ciphermemo;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.DigestException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;


//同期用のオブジェクト
final class LockObject extends Object {
    private static final LockObject instance = new LockObject();

    public static LockObject getInstance() {
        return instance;
    }
}

/**
 * メインクラス
 * @author kanemitsu
 */
public class JFrame_Main extends javax.swing.JFrame implements Runnable{
    private static final Color FOCUS_GAINED_COLOR = new Color(0xFFFFDF);
    private static final Color FOCUS_LOST_COLOR = new Color(0xFAFAFA);

    private boolean selectionEventEnabled = true;

    private long timeLimit = 0;
    
    //鍵の照合に使用する
    private byte[] KeyHash;

    //鍵を無効にするためのスレッド
    private Thread keyThread = new Thread(this);

    //設定の読み書きを行うクラス
    private IniFile ini = new IniFile();

    private boolean terminated = false;

    private long beforeKeyPressTime = 0;
    private long beforeMouseClickTime = 0;

    /**
     * ブロックごとにハッシュ値を計算する。
     * @param data ハッシュ値の計算対象
     * @return 連結したハッシュ値。
     * @throws NoSuchAlgorithmException
     * @throws DigestException
     */
    private byte[] createHash(byte[] data) throws NoSuchAlgorithmException, DigestException {
        //**変更可能**//
        MessageDigest md = MessageDigest.getInstance("SHA-512");
        int blockSize = 256;
        //************//

        int digestSize = md.getDigestLength();
        int pos = 0;
        int n = data.length / blockSize;
        byte[] result = new byte[Math.max(n, 1) * digestSize];
        
        for (int i = 0, i2 = 0; i < n; ++i) {
            if (i < n - 1) md.update(data, pos, blockSize);
            else md.update(data, pos, data.length - pos);
            
            for (byte b : md.digest()) {
                result[i2] = b;
                i2++;
            }
            pos += blockSize;
        }
        
        return result;
    }
    
    /**
     * 認証するだけで、鍵は破棄する。
     * @param parent
     * @return 鍵が有効ならtrue
     */
    private boolean showKeyInputDialog_ConfirmOnly() {
        //鍵が有効期限内
        if (this.timeLimit > 0) return true;
        
        byte[] key = showKeyInputDialog();
        if (key == null) {
            return false;
        }
        else {
            Arrays.fill(key, (byte)0);
            return true;
        }
    }
    
    /**
     * 鍵を入力するダイアログを表示する。
     * @param parent
     * @return 鍵が有効なら鍵を返す
     */
    private byte[] showKeyInputDialog() {
        synchronized (LockObject.getInstance()) {
            try {
                byte[] tempKey = null;
                byte[] tempkeyHash = null;

                while (true) {
                    //鍵の入力を求める
                    tempKey = jDialog_KeyInput.showKeyInputDialog(this);
                    if (tempKey == null) return null;

                    //照合用のハッシュ値を計算する
                    tempkeyHash = createHash(tempKey);

                    //初回なら照合用に設定
                    if (this.KeyHash == null) {
                        this.KeyHash = tempkeyHash;
                        break;
                    }

                    //照合する
                    else if (!Arrays.equals(tempkeyHash, this.KeyHash)){
                        Arrays.fill(tempKey, (byte)0);
                        JOptionPane.showMessageDialog(this, "鍵が違います。\n再度正しく入力して下さい。");
                    }

                    //ループを抜ける
                    else {
                        break;
                    }
                }

                timeLimit = jDialog_KeyInput.getTimeLimit();

                //まだ開始していなければ、開始する
                if (this.keyThread.getState() == Thread.State.NEW) this.keyThread.start();

                return tempKey;
            } catch (Exception ex) {
                Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                JOptionPane.showMessageDialog(this, ex);
            }
        }
        return null;
    }

    /**
     * 指定したノードの子ノードの名前と同じ物があるか調べる
     * @param node
     * @return 同じ物がなければtrueを返す
     */
    private boolean checkName(DefaultMutableTreeNode node, String name) {
        if (node.getChildCount() > 0) {
            for (int i = 0; i < node.getChildCount(); ++i) {
                if (((Memo)((DefaultMutableTreeNode)node.getChildAt(i)).getUserObject()).getName().equals(name)) {
                    return false;
                }
            }
        }
        return true;
    }

    private void saveNode(DefaultMutableTreeNode node, OutputStream stream, Cipher cipher) throws IOException {
        byte[] data;
        Memo memo = (Memo)node.getUserObject();
        memo.saveToStream(stream, cipher);

        data = new byte[4];
        if (node.isLeaf()) {
            Cipher.convertToByteArray(0, data);
        }
        else {
            Cipher.convertToByteArray(node.getChildCount(), data);
        }

        //子ノードの数を出力
        cipher.crypt(data);
        stream.write(data);

        if (!node.isLeaf()) {
            for (int i = 0, n = node.getChildCount(); i < n; ++i) {
                saveNode((DefaultMutableTreeNode) node.getChildAt(i), stream, cipher);
            }
        }
    }
    
    private void loadNode(DefaultMutableTreeNode node, InputStream stream, Cipher cipher) throws IOException {
        byte[] data;
        byte[] len = new byte[4];

        node.setUserObject(new Memo(stream, cipher));
        
        //子ノードの数を入力
        stream.read(len);
        cipher.crypt(len);
        int tempLength = Cipher.convertToInt(len);

        if (tempLength == 0) {
            //葉ノードなら探索終了
            return;
        }
        //サイズがおかしい場合は例外を発生させる
        else if ((tempLength < 0) || (tempLength > stream.available())) {
            throw new IOException("パスワードが違う可能性があります。");
        }
        else {
            //子ノードを読み込む
            for (int i = 0; i < tempLength; ++i) {
                DefaultMutableTreeNode child = new DefaultMutableTreeNode();
                node.add(child);
                loadNode(child , stream, cipher);
            }
        }

    }

    private void expandAll() {
        for (int i = 0, n = jTree_Category.getRowCount(); i < n; ++i) {
            jTree_Category.expandRow(i);
        }
    }

    private void expandNode(DefaultMutableTreeNode node) {
        if (node.isLeaf()) return;

        DefaultTreeModel model = (DefaultTreeModel) jTree_Category.getModel();
        TreePath path = new TreePath(model.getPathToRoot(node));
        jTree_Category.expandPath(path);
        for (int i = 0, n = node.getChildCount(); i < n; ++i) {
            expandNode((DefaultMutableTreeNode) node.getChildAt(i));
        }
    }

    private void addNode(DefaultMutableTreeNode parent) {
        DefaultTreeModel model = (DefaultTreeModel) jTree_Category.getModel();
        String name = "";

        //有効な名前が入力されるまでループ
        while (true) {
            name = JOptionPane.showInputDialog(this, "このメモの名前を入力して下さい。");
            if ((name == null) || name.isEmpty()) return;

            //重複しなければループを抜ける
            if (checkName(parent, name)) break;
            else JOptionPane.showMessageDialog(this, name + "はすでに存在する名前です。\n違う名前を入力して下さい。");
        }


        Memo memo = new Memo();
        memo.setName(name);

        if ((jTextArea_Memo.getText() != null) && (jTextArea_Memo.getText().length() > 0)) {

            try {
                synchronized (LockObject.getInstance()) {
                    if (!showKeyInputDialog_ConfirmOnly()) return;

                    //メモの内容を暗号化して設定
                    memo.setData(jTextArea_Memo.getText());

                }
            } catch (Exception ex) {
                Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                JOptionPane.showMessageDialog(this, ex);
                return;
            }
        }

        selectionEventEnabled = false;
        DefaultMutableTreeNode node = new DefaultMutableTreeNode(memo);
        parent.add(node);
        model.reload();
        jTree_Category.setSelectionPath(new TreePath(model.getPathToRoot(node)));
        selectionEventEnabled = true;

    }

    /** Creates new form NewJFrame */
    public JFrame_Main() {
        initComponents();

        setLocationRelativeTo(null);
        ((DefaultMutableTreeNode)jTree_Category.getModel().getRoot()).setAllowsChildren(true);

        ini.FileName = PathUtil.CURRENT_PATH + "setting.ini";

        if (new File(ini.FileName).exists()) {
            if (ini.LoadFile()) {
                String section = "UI";
                jTextArea_Memo.setFont(new Font(ini.ReadData(section, "FontName", "", false),
                                                ini.ReadData(section, "FontStyle", 0, false), 
                                                ini.ReadData(section, "FontSize", 16, false)));
            }
            else {
                JOptionPane.showMessageDialog(null, ini.FileName + " の読み込みに失敗しました。");
            }
        }

        jTree_Category.setBackground(FOCUS_LOST_COLOR);
        jTextArea_Memo.setBackground(FOCUS_LOST_COLOR);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTree_Category = new javax.swing.JTree();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextArea_Memo = new javax.swing.JTextArea();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu_File = new javax.swing.JMenu();
        jMenuItem_File_Open = new javax.swing.JMenuItem();
        jMenuItem_File_Save = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        jMenuItem_File_Exit = new javax.swing.JMenuItem();
        jMenu_Edit = new javax.swing.JMenu();
        jMenu_Edit_AddMemo = new javax.swing.JMenu();
        jMenuItem_Edit_AddMemo_Sibling = new javax.swing.JMenuItem();
        jMenuItem_Edit_AddMemo_Child = new javax.swing.JMenuItem();
        jMenuItem_Edit_DeleteMemo = new javax.swing.JMenuItem();
        jMenuItem_Edit_UpdateMemo = new javax.swing.JMenuItem();
        jMenuItem_Edit_Rename = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JPopupMenu.Separator();
        jMenuItem_Edit_KeyDisable = new javax.swing.JMenuItem();
        jMenuItem_Edit_Font = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("CipherMemo");
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                formMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                formMouseReleased(evt);
            }
        });
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                formMouseMoved(evt);
            }
        });
        addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                formKeyPressed(evt);
            }
        });

        jSplitPane1.setDividerLocation(200);

        jTree_Category.setBackground(new java.awt.Color(250, 250, 250));
        jTree_Category.setFont(jTree_Category.getFont().deriveFont((float)16));
        javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("root");
        jTree_Category.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        jTree_Category.setToolTipText("<html>メモを階層構造で管理する、「ツリー」です。<br>\n<br>\nダブルクリック、または選択状態で「←」「→」キーで開閉できます。");
        jTree_Category.setAutoscrolls(true);
        jTree_Category.setDoubleBuffered(true);
        jTree_Category.setRootVisible(false);
        jTree_Category.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                jTree_CategoryValueChanged(evt);
            }
        });
        jTree_Category.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                jTree_CategoryFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTree_CategoryFocusLost(evt);
            }
        });
        jScrollPane1.setViewportView(jTree_Category);

        jSplitPane1.setLeftComponent(jScrollPane1);

        jTextArea_Memo.setBackground(new java.awt.Color(250, 250, 250));
        jTextArea_Memo.setColumns(20);
        jTextArea_Memo.setFont(jTextArea_Memo.getFont().deriveFont((float)16));
        jTextArea_Memo.setRows(5);
        jTextArea_Memo.setToolTipText("<html>メモの内容を編集する「テキストボックス」です。<br>\n<br>\n\"Ctrl + X\" - 切り取り<br>\n\"Ctrl + C\" - コピー<br>\n\"Ctrl + V\" - 貼り付け<br>\n\"Ctrl + A\" - 全て選択");
        jTextArea_Memo.setDoubleBuffered(true);
        jTextArea_Memo.setDragEnabled(true);
        jTextArea_Memo.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                jTextArea_MemoFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextArea_MemoFocusLost(evt);
            }
        });
        jScrollPane2.setViewportView(jTextArea_Memo);

        jSplitPane1.setRightComponent(jScrollPane2);

        jMenuBar1.setFont(jMenuBar1.getFont().deriveFont((float)19));

        jMenu_File.setText("ファイル");
        jMenu_File.setFont(jMenu_File.getFont().deriveFont((float)19));

        jMenuItem_File_Open.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_File_Open.setFont(jMenuItem_File_Open.getFont().deriveFont((float)19));
        jMenuItem_File_Open.setText("開く");
        jMenuItem_File_Open.setToolTipText("メモをファイルから読み込みます。");
        jMenuItem_File_Open.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_File_OpenActionPerformed(evt);
            }
        });
        jMenu_File.add(jMenuItem_File_Open);

        jMenuItem_File_Save.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_File_Save.setFont(jMenuItem_File_Save.getFont().deriveFont((float)19));
        jMenuItem_File_Save.setText("保存");
        jMenuItem_File_Save.setToolTipText("メモをファイルに保存します。");
        jMenuItem_File_Save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_File_SaveActionPerformed(evt);
            }
        });
        jMenu_File.add(jMenuItem_File_Save);
        jMenu_File.add(jSeparator1);

        jMenuItem_File_Exit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_File_Exit.setFont(jMenuItem_File_Exit.getFont().deriveFont((float)19));
        jMenuItem_File_Exit.setText("終了");
        jMenuItem_File_Exit.setToolTipText("プログラムを終了します。");
        jMenuItem_File_Exit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_File_ExitActionPerformed(evt);
            }
        });
        jMenu_File.add(jMenuItem_File_Exit);

        jMenuBar1.add(jMenu_File);

        jMenu_Edit.setText("編集");
        jMenu_Edit.setFont(jMenu_Edit.getFont().deriveFont((float)19));

        jMenu_Edit_AddMemo.setText("メモを追加");

        jMenuItem_Edit_AddMemo_Sibling.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_Edit_AddMemo_Sibling.setFont(jMenuItem_Edit_AddMemo_Sibling.getFont().deriveFont((float)19));
        jMenuItem_Edit_AddMemo_Sibling.setText("同じ階層");
        jMenuItem_Edit_AddMemo_Sibling.setToolTipText("ツリーで選択されたメモと同じ階層に、メモを追加します。");
        jMenuItem_Edit_AddMemo_Sibling.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_Edit_AddMemo_SiblingActionPerformed(evt);
            }
        });
        jMenu_Edit_AddMemo.add(jMenuItem_Edit_AddMemo_Sibling);

        jMenuItem_Edit_AddMemo_Child.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_Edit_AddMemo_Child.setText("1つ下の階層");
        jMenuItem_Edit_AddMemo_Child.setToolTipText("ツリーで選択されたメモの1つ下の階層に、メモを追加します。");
        jMenuItem_Edit_AddMemo_Child.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_Edit_AddMemo_ChildActionPerformed(evt);
            }
        });
        jMenu_Edit_AddMemo.add(jMenuItem_Edit_AddMemo_Child);

        jMenu_Edit.add(jMenu_Edit_AddMemo);

        jMenuItem_Edit_DeleteMemo.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_Edit_DeleteMemo.setFont(jMenuItem_Edit_DeleteMemo.getFont().deriveFont((float)19));
        jMenuItem_Edit_DeleteMemo.setText("メモを削除");
        jMenuItem_Edit_DeleteMemo.setToolTipText("<html>ツリーで選択されたメモと、その下の階層にあるメモ全てを削除します。");
        jMenuItem_Edit_DeleteMemo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_Edit_DeleteMemoActionPerformed(evt);
            }
        });
        jMenu_Edit.add(jMenuItem_Edit_DeleteMemo);

        jMenuItem_Edit_UpdateMemo.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_U, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_Edit_UpdateMemo.setFont(jMenuItem_Edit_UpdateMemo.getFont().deriveFont((float)19));
        jMenuItem_Edit_UpdateMemo.setText("メモを上書き");
        jMenuItem_Edit_UpdateMemo.setToolTipText("<html>ツリーで選択されたメモの内容を、テキストボックスで編集された内容に更新します。");
        jMenuItem_Edit_UpdateMemo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_Edit_UpdateMemoActionPerformed(evt);
            }
        });
        jMenu_Edit.add(jMenuItem_Edit_UpdateMemo);

        jMenuItem_Edit_Rename.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_Edit_Rename.setFont(jMenuItem_Edit_Rename.getFont().deriveFont((float)19));
        jMenuItem_Edit_Rename.setText("メモの名前を変更");
        jMenuItem_Edit_Rename.setToolTipText("ツリーで選択されたメモの名前を変更します。");
        jMenuItem_Edit_Rename.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_Edit_RenameActionPerformed(evt);
            }
        });
        jMenu_Edit.add(jMenuItem_Edit_Rename);
        jMenu_Edit.add(jSeparator2);

        jMenuItem_Edit_KeyDisable.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_K, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_Edit_KeyDisable.setFont(jMenuItem_Edit_KeyDisable.getFont().deriveFont((float)19));
        jMenuItem_Edit_KeyDisable.setText("鍵を無効にする");
        jMenuItem_Edit_KeyDisable.setToolTipText("<html>設定した有効期間を無視して、今すぐ鍵を無効にします。");
        jMenuItem_Edit_KeyDisable.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_Edit_KeyDisableActionPerformed(evt);
            }
        });
        jMenu_Edit.add(jMenuItem_Edit_KeyDisable);

        jMenuItem_Edit_Font.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem_Edit_Font.setText("フォントの変更");
        jMenuItem_Edit_Font.setToolTipText("<html>メモの内容を表示しているテキストボックスのフォントを変更します。");
        jMenuItem_Edit_Font.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem_Edit_FontActionPerformed(evt);
            }
        });
        jMenu_Edit.add(jMenuItem_Edit_Font);

        jMenuBar1.add(jMenu_Edit);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 573, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 366, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jMenuItem_Edit_AddMemo_SiblingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_Edit_AddMemo_SiblingActionPerformed
        
        DefaultTreeModel model = (DefaultTreeModel) jTree_Category.getModel();
        DefaultMutableTreeNode selectedNode = null;
        
        if (jTree_Category.isSelectionEmpty()) selectedNode = (DefaultMutableTreeNode) model.getRoot();
        else selectedNode = (DefaultMutableTreeNode) jTree_Category.getSelectionPath().getParentPath().getLastPathComponent();

        addNode(selectedNode);
    }//GEN-LAST:event_jMenuItem_Edit_AddMemo_SiblingActionPerformed

    private void jTree_CategoryValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_jTree_CategoryValueChanged
        
        if (selectionEventEnabled && (!jTree_Category.isSelectionEmpty())) {
            
            
            TreePath path = null;

            try {
                //新たに選択されたパスがないか調べる。
                if (!evt.isAddedPath(jTree_Category.getSelectionPath())) return;
            } catch (Exception ex) {
                return;
            }
            
            Memo memo = (Memo) ((DefaultMutableTreeNode)jTree_Category.getSelectionPath().getLastPathComponent()).getUserObject();

            //メモの中身がなければ、TextAreaを初期化する
            if (memo.isEmpty()) {
                jTextArea_Memo.setText("");
                return;
            }
            
            synchronized (LockObject.getInstance()) {
                if (showKeyInputDialog_ConfirmOnly()) {
                    try {

                        //メモの内容を復号して表示
                        jTextArea_Memo.setText(memo.getPlaneData());

                    } catch (Exception ex) {
                        Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                        JOptionPane.showMessageDialog(this, ex);
                        return;
                    }
                }
            }
            
        }
    }//GEN-LAST:event_jTree_CategoryValueChanged

    private void jMenuItem_File_ExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_File_ExitActionPerformed
        formWindowClosing(null);
    }//GEN-LAST:event_jMenuItem_File_ExitActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        //終了確認をする
        if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(this, "終了してもよろしいですか？", "確認", JOptionPane.YES_NO_OPTION)) {
            //スレッドが実行中なら終了させる
            if (keyThread.getState() == Thread.State.RUNNABLE) {
                terminated = true;
                keyThread.interrupt();
                try {
                    keyThread.join(1000);
                } catch (InterruptedException ex) {
                    Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                }
                keyThread = null;
            }
            System.exit(0);
        }
    }//GEN-LAST:event_formWindowClosing

    private void jMenuItem_Edit_RenameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_Edit_RenameActionPerformed
        if (jTree_Category.isSelectionEmpty()) {
            JOptionPane.showMessageDialog(this, "名前を変更するメモを選択して下さい。");
            return;
        }
        
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)jTree_Category.getSelectionPath().getLastPathComponent();
        String name;

        //有効な名前が入力されるまでループ
        while (true) {
            name = JOptionPane.showInputDialog(this, "このメモの名前を入力して下さい。");
            if ((name == null) || name.isEmpty()) return;

            //重複しなければループを抜ける
            if (checkName((DefaultMutableTreeNode) node.getParent(), name)) break;
            else JOptionPane.showMessageDialog(this, name + "はすでに存在する名前です。\n違う名前を入力して下さい。");
        }

        ((Memo)node.getUserObject()).setName(name);
        ((DefaultTreeModel)jTree_Category.getModel()).reload(node);
    }//GEN-LAST:event_jMenuItem_Edit_RenameActionPerformed

    private void jMenuItem_Edit_DeleteMemoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_Edit_DeleteMemoActionPerformed
        if (jTree_Category.isSelectionEmpty()) {
            JOptionPane.showMessageDialog(this, "削除するメモを選択して下さい。");
            return;
        }

        if (JOptionPane.NO_OPTION == JOptionPane.showConfirmDialog(this, "本当に削除してもよろしいですか？", "確認", JOptionPane.YES_NO_OPTION)) return;
        if (!showKeyInputDialog_ConfirmOnly()) return;
        
        DefaultTreeModel model = (DefaultTreeModel) jTree_Category.getModel();
        for (TreePath path : jTree_Category.getSelectionPaths()) {
            try {
                ((DefaultMutableTreeNode)path.getLastPathComponent()).removeFromParent();
            } catch (Exception ex) {

            }

        }
        model.reload();
        expandNode((DefaultMutableTreeNode) model.getRoot());
    }//GEN-LAST:event_jMenuItem_Edit_DeleteMemoActionPerformed

    private void jMenuItem_Edit_UpdateMemoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_Edit_UpdateMemoActionPerformed
        if (jTree_Category.isSelectionEmpty()) {
            JOptionPane.showMessageDialog(this, "メモが選択されていません。");
            return;
        }

        Memo memo = (Memo) ((DefaultMutableTreeNode)jTree_Category.getSelectionPath().getLastPathComponent()).getUserObject();

        if ((jTextArea_Memo.getText() != null) && (jTextArea_Memo.getText().length() > 0)) {
            synchronized (LockObject.getInstance()) {
                if (showKeyInputDialog_ConfirmOnly()) {
                    try {

                        //メモの内容を暗号化して設定
                        memo.setData(jTextArea_Memo.getText());

                    } catch (Exception ex) {
                        Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                        JOptionPane.showMessageDialog(this, ex);
                        return;
                    }
                }
            }
        }
        else if (!memo.isEmpty()){
            if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(this, "メモの内容を消去してもよろしいですか？", "確認", JOptionPane.YES_NO_OPTION)) {
                try {
                    memo.setEmpty();
                } catch (Exception ex) {
                    Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                    JOptionPane.showMessageDialog(this, ex);
                }
            }
        }
        
    }//GEN-LAST:event_jMenuItem_Edit_UpdateMemoActionPerformed

    private void jMenuItem_Edit_KeyDisableActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_Edit_KeyDisableActionPerformed
        synchronized (LockObject.getInstance()) {
            //鍵を無効にする
            this.timeLimit = 0;
        }
    }//GEN-LAST:event_jMenuItem_Edit_KeyDisableActionPerformed

    private void jMenuItem_File_SaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_File_SaveActionPerformed
        JFileChooser chooser = new JFileChooser();
        chooser.setAcceptAllFileFilterUsed(true);
        chooser.setMultiSelectionEnabled(false);
        if (JFileChooser.APPROVE_OPTION != chooser.showSaveDialog(this)) return;

        synchronized (LockObject.getInstance()) {

            //鍵を求める
            byte[] key = showKeyInputDialog();
            if (key == null) return;

            Cipher cipher = new Cipher(key);
           
            FileOutputStream fos = null;
            BufferedOutputStream bos = null;
            try {
                fos = new FileOutputStream(chooser.getSelectedFile());
                bos = new BufferedOutputStream(fos);
                
                DefaultMutableTreeNode root = (DefaultMutableTreeNode)jTree_Category.getModel().getRoot();
                int len = root.getChildCount();
                byte[] data = null;
                        
                //鍵のハッシュを出力
                /*
                byte[] data = Arrays.copyOf(tempKeyHash, 4);
                cipher.crypt(data);
                bos.write(data);
                 * 
                 */

                //rootの子ノードの数を出力
                data = new byte[4];
                Cipher.convertToByteArray(len, data);
                cipher.crypt(data);
                bos.write(data);
                
                for (int i = 0; i < len; ++i) {
                    saveNode((DefaultMutableTreeNode) root.getChildAt(i), bos, cipher);
                }
                
                bos.flush();
            } catch (IOException ex) {
                Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                JOptionPane.showMessageDialog(this, ex);
            } finally {
                try {
                    fos.close();
                    bos.close();
                } catch (IOException ex) {
                    Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                    JOptionPane.showMessageDialog(this, ex);
                }

            }
        }


    }//GEN-LAST:event_jMenuItem_File_SaveActionPerformed

    private void jMenuItem_File_OpenActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_File_OpenActionPerformed
        JFileChooser chooser = new JFileChooser();
        chooser.setAcceptAllFileFilterUsed(true);
        chooser.setMultiSelectionEnabled(false);
        if (JFileChooser.APPROVE_OPTION != chooser.showOpenDialog(this)) return;

        synchronized (LockObject.getInstance()) {

            //鍵を求める
            byte[] key = showKeyInputDialog();
            if (key == null) return;

            //暗号クラスを生成する
            Cipher cipher = new Cipher(key);
            
            FileInputStream fis = null;
            BufferedInputStream bis = null;
            try {
                fis = new FileInputStream(chooser.getSelectedFile());
                bis = new BufferedInputStream(fis);
                
                byte[] data = new byte[4];

                //鍵のハッシュを入力
                /*
                bis.read(data);
                cipher.crypt(data);
                if (!Arrays.equals(data, this.tempKeyHash)) {
                    if (JOptionPane.NO_OPTION == JOptionPane.showConfirmDialog(this, "鍵が違う可能性があります。\nこのまま続行しますか？", "確認", JOptionPane.YES_NO_OPTION)) {
                        jMenuItem_Edit_KeyDisableActionPerformed(null);
                        this.tempKeyHash = null;
                        return;
                    }
                }
                 *
                 */

                //rootの子ノードの数を入力
                bis.read(data);
                cipher.crypt(data);
                int len = Cipher.convertToInt(data);

                DefaultTreeModel model = (DefaultTreeModel) jTree_Category.getModel();
                DefaultMutableTreeNode root = (DefaultMutableTreeNode)model.getRoot();

                //ツリーを初期化する
                root.removeAllChildren();
                jTextArea_Memo.setText("");

                for (int i = 0; i < len; ++i) {
                    DefaultMutableTreeNode child = new DefaultMutableTreeNode();
                    root.add(child);
                    loadNode(child, bis, cipher);
                }

                ((DefaultTreeModel)jTree_Category.getModel()).reload();
                expandNode(root);
            } catch (Exception ex) {
                Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                JOptionPane.showMessageDialog(this, ex);
            } finally {
                try {
                    fis.close();
                    bis.close();
                } catch (IOException ex) {
                    Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
                    JOptionPane.showMessageDialog(this, ex);
                }
            }
        }

    }//GEN-LAST:event_jMenuItem_File_OpenActionPerformed

    private void formKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyPressed
        long time = System.currentTimeMillis();
        TrueRandom.addEntropyData(time - this.beforeKeyPressTime);
        this.beforeKeyPressTime = time;

        if ((evt.getModifiers() & (KeyEvent.SHIFT_DOWN_MASK | KeyEvent.CTRL_DOWN_MASK)) != 0) {
            switch (evt.getKeyCode()) {
                case KeyEvent.VK_A: {
                    jMenuItem_Edit_AddMemo_SiblingActionPerformed(null);
                    break;
                }
                case KeyEvent.VK_C: {
                    jMenuItem_Edit_AddMemo_ChildActionPerformed(null);
                    break;
                }
                case KeyEvent.VK_D: {
                    jMenuItem_Edit_DeleteMemoActionPerformed(null);
                    break;
                }
                case KeyEvent.VK_R: {
                    jMenuItem_Edit_RenameActionPerformed(null);
                    break;
                }
                case KeyEvent.VK_U: {
                    jMenuItem_Edit_UpdateMemoActionPerformed(null);
                    break;
                }
                case KeyEvent.VK_K: {
                    jMenuItem_Edit_KeyDisableActionPerformed(null);
                    break;
                }
                
                case KeyEvent.VK_O: {
                    jMenuItem_File_OpenActionPerformed(null);
                    break;
                }
                case KeyEvent.VK_S: {
                    jMenuItem_File_SaveActionPerformed(null);
                    break;
                }
                case KeyEvent.VK_X: {
                    jMenuItem_File_ExitActionPerformed(null);
                    break;
                }
            }
        }
    }//GEN-LAST:event_formKeyPressed

    private void jMenuItem_Edit_AddMemo_ChildActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_Edit_AddMemo_ChildActionPerformed
        DefaultTreeModel model = (DefaultTreeModel) jTree_Category.getModel();
        DefaultMutableTreeNode selectedNode = null;

        if (jTree_Category.isSelectionEmpty()) selectedNode = (DefaultMutableTreeNode) model.getRoot();
        else selectedNode = (DefaultMutableTreeNode) jTree_Category.getSelectionPath().getLastPathComponent();

        addNode(selectedNode);
    }//GEN-LAST:event_jMenuItem_Edit_AddMemo_ChildActionPerformed

    private void jMenuItem_Edit_FontActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem_Edit_FontActionPerformed
        FontDialog fd = new FontDialog(jTextArea_Memo.getFont());
        Font font = fd.showDialog();

        if (font != null) {
            String section = "UI";
            jTextArea_Memo.setFont(font);

            this.ini.WriteData(section, "FontName", font.getFamily(), false);
            this.ini.WriteData(section, "FontStyle", font.getStyle(), false);
            this.ini.WriteData(section, "FontSize", font.getSize(), false);
            this.ini.SaveFile();
        }

    }//GEN-LAST:event_jMenuItem_Edit_FontActionPerformed

    private void jTextArea_MemoFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextArea_MemoFocusGained
        jTextArea_Memo.setBackground(FOCUS_GAINED_COLOR);
    }//GEN-LAST:event_jTextArea_MemoFocusGained

    private void jTextArea_MemoFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextArea_MemoFocusLost
        jTextArea_Memo.setBackground(FOCUS_LOST_COLOR);
    }//GEN-LAST:event_jTextArea_MemoFocusLost

    private void jTree_CategoryFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTree_CategoryFocusGained
        jTree_Category.setBackground(FOCUS_GAINED_COLOR);
    }//GEN-LAST:event_jTree_CategoryFocusGained

    private void jTree_CategoryFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTree_CategoryFocusLost
        jTree_Category.setBackground(FOCUS_LOST_COLOR);
    }//GEN-LAST:event_jTree_CategoryFocusLost

    private void formMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseMoved
        TrueRandom.addEntropyData(evt.getX());
        TrueRandom.addEntropyData(evt.getY());
    }//GEN-LAST:event_formMouseMoved

    private void formMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMousePressed
        long time = System.currentTimeMillis();
        TrueRandom.addEntropyData(time - this.beforeMouseClickTime);
        TrueRandom.addEntropyData((byte)((evt.getButton() + 1) * evt.getClickCount()));

        this.beforeMouseClickTime = time;
    }//GEN-LAST:event_formMousePressed

    private void formMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseReleased
        formMousePressed(evt);
    }//GEN-LAST:event_formMouseReleased
    
    //別スレッドによって、1秒ごとにtimeLimitをカウントダウンする
    public void run() {
        while (!this.terminated) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException ex) {
                Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
            }
            
            synchronized (LockObject.getInstance()) {
                if (this.timeLimit > 0) {
                    this.timeLimit -= 1000;
                    
                }
            }
        }
    }

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        UIManager.getInstalledLookAndFeels();
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
        } catch (UnsupportedLookAndFeelException ex) {
            Logger.getLogger(JFrame_Main.class.getName()).log(Level.SEVERE, null, ex);
        }

        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new JFrame_Main().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem_Edit_AddMemo_Child;
    private javax.swing.JMenuItem jMenuItem_Edit_AddMemo_Sibling;
    private javax.swing.JMenuItem jMenuItem_Edit_DeleteMemo;
    private javax.swing.JMenuItem jMenuItem_Edit_Font;
    private javax.swing.JMenuItem jMenuItem_Edit_KeyDisable;
    private javax.swing.JMenuItem jMenuItem_Edit_Rename;
    private javax.swing.JMenuItem jMenuItem_Edit_UpdateMemo;
    private javax.swing.JMenuItem jMenuItem_File_Exit;
    private javax.swing.JMenuItem jMenuItem_File_Open;
    private javax.swing.JMenuItem jMenuItem_File_Save;
    private javax.swing.JMenu jMenu_Edit;
    private javax.swing.JMenu jMenu_Edit_AddMemo;
    private javax.swing.JMenu jMenu_File;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JPopupMenu.Separator jSeparator2;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTextArea jTextArea_Memo;
    private javax.swing.JTree jTree_Category;
    // End of variables declaration//GEN-END:variables

}
