/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ciphermemo;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;

/**
 * 個々のメモを格納するクラス
 * @author kanemitsu
 */
public class Memo {

    /**
     * メモリの暗号化に使用する鍵
     */
    private static final int KEY_SIZE = 4 * 8;
    private byte[] key;
    private byte[] key_key;
    private int randomNumber;

    private byte[] data;
    private String name = "";

    public Memo() {
    }

    /**
     * コンストラクタ
     * @param stream 読み込み先のストリーム
     * @param cipher 復号に使用するクラス
     */
    public Memo(InputStream stream, Cipher cipher) throws IOException {
        byte[] data = null;
        byte[] len = new byte[4];
        int tempLength = 0;
        
        //真の乱数を入力
        stream.read(len);
        cipher.crypt(len);
        this.randomNumber = Cipher.convertToInt(len);

        //乱数を捨てる
        cipher.skip(this.randomNumber & 0xFF);

        //乱数1ブロックのサイズを設定
        cipher.setRandomBlockLength((this.randomNumber & 0xFF00) >>> 8);

        //鍵のサイズを入力
        stream.read(len);
        cipher.crypt(len);
        tempLength = Cipher.convertToInt(len);
        
        //サイズがおかしい場合は例外を発生させる
        if ((tempLength < 0) || (tempLength > stream.available())) {
            throw new IOException("パスワードが違う可能性があります。");
        } 
        
        if (tempLength > 0) {
            //鍵を入力
            data = new byte[tempLength];
            stream.read(data);
            cipher.crypt(data);
            this.key = data;
            this.key_key = new byte[KEY_SIZE];
            TrueRandom.getValueArray(this.key_key);
            cryptKey();
        }
        
        //名前の長さを入力
        stream.read(len);
        cipher.crypt(len);
        tempLength = Cipher.convertToInt(len);
        
        //サイズがおかしい場合は例外を発生させる
        if ((tempLength < 0) || (tempLength > stream.available())) {
            throw new IOException("パスワードが違う可能性があります。");
        } 

        if (tempLength > 0) {
            //名前を入力
            data = new byte[tempLength];
            stream.read(data);
            cipher.crypt(data);
            this.name = (new String(data, "UTF8"));
        }
        //メモの長さを入力
        stream.read(len);
        cipher.crypt(len);
        tempLength = Cipher.convertToInt(len);
        
        //サイズがおかしい場合は例外を発生させる
        if ((tempLength < 0) || (tempLength > stream.available())) {
            throw new IOException("パスワードが違う可能性があります。");
        }
        else if (tempLength > 0) {
            //メモを入力
            data = new byte[tempLength];
            stream.read(data);
            cipher.crypt(data);
            this.data = data;
        }
        else {
            this.data = null;
        }

    }

    public void saveToStream(OutputStream stream, Cipher cipher) throws IOException {
        byte[] data = null;
        byte[] len = new byte[4];
        
        //真の乱数を出力
        Cipher.convertToByteArray(this.randomNumber, len);
        cipher.crypt(len);
        stream.write(len);

        //乱数を捨てる
        cipher.skip(this.randomNumber & 0xFF);

        //乱数1ブロックのサイズを設定
        cipher.setRandomBlockLength((this.randomNumber & 0xFF00) >>> 8);

        //鍵のサイズを出力
        if (this.key == null) Arrays.fill(len, (byte)0);
        else Cipher.convertToByteArray(this.key.length, len);
        cipher.crypt(len);
        stream.write(len);
        
        if (this.key != null) {
            //鍵を出力
            data = Arrays.copyOf(this.key, this.key.length);
            cryptKey();
            cipher.crypt(this.key);
            stream.write(this.key);
            this.key = data;
        }
        
        if (!this.name.isEmpty()) {
            data = this.name.getBytes("UTF8");
            Cipher.convertToByteArray(data.length, len);
        }
        else {
            Arrays.fill(len, (byte)0);
        }
        
        //名前の長さを出力
        cipher.crypt(len);
        stream.write(len);

        if (!this.name.isEmpty()) {
            //名前を出力
            cipher.crypt(data);
            stream.write(data);
        }

        //メモの長さを出力
        if (this.data == null) Arrays.fill(len, (byte)0);
        else Cipher.convertToByteArray(this.data.length, len);
        cipher.crypt(len);
        stream.write(len);
        
        if (this.data != null) {
            //メモを出力
            data = Arrays.copyOf(this.data, this.data.length);
            cipher.crypt(data);
            stream.write(data);
        }
    }

    public byte[] getKey() {
        byte[] result = Arrays.copyOf(this.key, this.key.length);
        Cipher cipher = new Cipher(this.key_key);
        cipher.skip(result[0]);
        cipher.setRandomBlockLength(result[0]);
        cipher.crypt(result);
        cipher.dispose();

        return result;
    }

    private void cryptKey() {
        Cipher cipher = new Cipher(this.key_key);
        cipher.skip(this.randomNumber);
        cipher.setRandomBlockLength(this.randomNumber);
        cipher.crypt(this.key);
        cipher.dispose();
    }
    
    public void initKey() {
        if (this.key != null) Arrays.fill(this.key, (byte)0);
        if (this.key_key != null) Arrays.fill(this.key_key, (byte)0);
        
        this.key = new byte[KEY_SIZE];
        this.key_key = new byte[KEY_SIZE];
        
        TrueRandom.getValueArray(key);
        TrueRandom.getValueArray(key_key);
        randomNumber = TrueRandom.getValue() & 0xFF;
        
        cryptKey();
    }
    
    public byte[] getData() {
        if (isEmpty()) return null;
        else return Arrays.copyOf(this.data, this.data.length);
        
    }

    /**
     * 暗号化したデータを復号して返す
     * @return 復号したデータ
     */
    public String getPlaneData() throws UnsupportedEncodingException {
        if (isEmpty()) return "";
        else {
            byte[] result = Arrays.copyOf(this.data, this.data.length);

            //鍵を復号
            cryptKey();

            Cipher cipher = new Cipher(this.key);
            cipher.skip((this.randomNumber & 0xFF0000) >>> 16);
            cipher.setRandomBlockLength((this.randomNumber & 0xFF000000) >>> 24);
            cipher.crypt(result);
            cipher.dispose();

            //鍵を暗号化
            cryptKey();
            
            return new String(result, "UTF8");
        }
        
    }

    public void setEmpty() {
        if (!isEmpty()) Arrays.fill(this.data, (byte)0);
        this.data = null;
        initKey();
    }
    
    /**
     * 本文を設定する
     * @param data 暗号化していない本文
     */
    public void setData(String data) throws UnsupportedEncodingException {
        
        byte[] b = data.getBytes("UTF8");

        //鍵を初期化
        initKey();
        
        //鍵を復号
        cryptKey();

        Cipher cipher = new Cipher(this.key);
        cipher.skip((this.randomNumber & 0xFF0000) >>> 16);
        cipher.setRandomBlockLength((this.randomNumber & 0xFF000000) >>> 24);
        cipher.crypt(b);
        cipher.dispose();

        //鍵を暗号化
        cryptKey();
        
        this.data = b;
    }

    /**
     * 本文を設定する
     * @param data 暗号化済みの本文
     */
    public void setData(byte[] data) {
        if (data == null) {
            this.data = null;
        }
        else {
            this.data = new byte[data.length];
            System.arraycopy(data, 0, this.data, 0, data.length);
        }
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        if (name == null) this.name = "";
        else this.name = name;
    }

    public boolean isEmpty() {
        return this.data == null;
    }
    
    @Override
    public String toString() {
        return this.name;
    }
    
}
