package ciphermemo;

/*
Sfmt.java 乱数ライブラリ(216091)
coded by isaku@pb4.so-net.ne.jp
*/

/*
使わない部分を削除。
マジックナンバーを定数に置き換え。
数字の意図が分からないので名前は適当。
edited by kanemitsu
*/

class Sfmt
{
    private int    index;
    private int    coin_bits; /* NextBit での残りのビット */
    private int    coin_save; /* NextBit での値保持 */
    private int    byte_pos;  /* NextByte で使用したバイト数 */
    private int    byte_save; /* NextByte での値保持 */
    private int    range; /* NextIntEx で前回の範囲 */
    private int    base; /* NextIntEx で前回の基準値 */
    private int    shift; /* NextIntEx で前回のシフト数 */
    private int    normal_sw; /* NextNormal で残りを持っている */
    private double normal_save; /* NextNormal の残りの値 */
    private static final int x_len = 6756; /* 状態テーブルのサイズ */
    private static final int x_len_minus_1 = x_len - 1;
    private static final int num_3372 = 3372;
    private static final int num_3383 = 3383;
    private static final int num_1664525 = 1664525;
    protected int[] x = new int[x_len]; /* 状態テーブル */

    protected int[]GetParity()
    { return new int[]{ 0xf8000001,0x89e80709,0x3bd2b64b,0x0c64b1e4 }; }

    public String IdString() {
        return"SFMT-216091:627-11-3-10-1:bff7bff7-bfffffff-bffffa7f-ffddfbfb";
    }

    public void gen_rand_all() {
        int a=0,b=2508,c=6748,d=6752,y; int[]p=x;

        do {
            y=p[a+3]^(p[a+3]<<24)^(p[a+2]>>>8)^((p[b+3]>>>10)&0xffddfbfb);
            p[a+3]=y^(p[c+3]>>>8)^(p[d+3]<<11);
            y=p[a+2]^(p[a+2]<<24)^(p[a+1]>>>8)^((p[b+2]>>>10)&0xbffffa7f);
            p[a+2]=y^((p[c+2]>>>8)|(p[c+3]<<24))^(p[d+2]<<11);
            y=p[a+1]^(p[a+1]<<24)^(p[a]>>>8)^((p[b+1]>>>10)&0xbfffffff);
            p[a+1]=y^((p[c+1]>>>8)|(p[c+2]<<24))^(p[d+1]<<11);
            y=p[a]^(p[a]<<24)^((p[b]>>>10)&0xbff7bff7);
            p[a]=y^((p[c]>>>8)|(p[c+1]<<24))^(p[d]<<11);
            c=d; d=a; a+=4; b+=4;
            if (b==x_len) b=0;
        } while (a!=x_len);
    }

    private void period_certification() {
        int work,inner=0; int i,j; int[] parity=GetParity();

        index=x_len; range=0; normal_sw=0; coin_bits=0; byte_pos=0;
        for (i=0;i<4;i++) inner^=x[i]&parity[i];
        for (i=16;i>0;i>>>=1) inner^=inner>>>i;
        inner&=1;
        if (inner==1) return;
        for (i=0;i<4;i++) for (j=0,work=1;j<32;j++,work<<=1)
         if ((work&parity[i])!=0) { x[i]^=work; return; }
    }

    /* 整数の種 s による初期化 */
    /*synchronized*/ public void InitMt(int s) {
        x[0]=s;
        for (int p=1;p<x_len;p++)
            x[p]=s=1812433253*(s^(s>>>30))+p;
        period_certification();
    }

    public Sfmt(int s) { InitMt(s); }

    /* 配列 init_key による初期化 */
    /*synchronized*/ public void InitMtEx(int[]init_key) {
        int r,i,j,c,key_len=init_key.length;

        for (i=0;i<x_len;i++) x[i]=0x8b8b8b8b;
        if (key_len+1>x_len) c=key_len+1; else c=x_len;
        r=x[0]^x[num_3372]^x[x_len_minus_1]; r=(r^(r>>>27))*num_1664525;
        x[num_3372]+=r; r+=key_len; x[num_3383]+=r; x[0]=r; c--;
        for (i=1,j=0;j<c&&j<key_len;j++) {
            r=x[i]^x[(i+num_3372)%x_len]^x[(i+x_len_minus_1)%x_len];
            r=(r^(r>>>27))*num_1664525; x[(i+num_3372)%x_len]+=r;
            r+=init_key[j]+i; x[(i+num_3383)%x_len]+=r;
            x[i]=r; i=(i+1)%x_len;
        }
        for (;j<c;j++) {
            r=x[i]^x[(i+num_3372)%x_len]^x[(i+x_len_minus_1)%x_len];
            r=(r^(r>>>27))*num_1664525; x[(i+num_3372)%x_len]+=r; r+=i;
            x[(i+num_3383)%x_len]+=r; x[i]=r; i=(i+1)%x_len;
        }
        for (j=0;j<x_len;j++) {
            r=x[i]+x[(i+num_3372)%x_len]+x[(i+x_len_minus_1)%x_len];
            r=(r^(r>>>27))*1566083941; x[(i+num_3372)%x_len]^=r;
            r-=i; x[(i+num_3383)%x_len]^=r; x[i]=r; i=(i+1)%x_len;
        }
        period_certification();
    }

    public Sfmt(int[]init_key) { InitMtEx(init_key); }

    /* 32ビット符号あり整数の乱数 */
    /*synchronized*/ public int NextMt() {
        if (index==x.length) { gen_rand_all(); index=0; }
        return x[index++];
    }

    /* ０以上 n 未満の整数乱数 */
    public int NextInt(int n) {
        double z=NextMt();
        if (z<0) z+=4294967296.0;
        return(int)(n*(1.0/4294967296.0)*z);
    }

    /* ０以上１未満の乱数(53bit精度) */
    public double NextUnif() {
        double z=NextMt()>>>11,y=NextMt();
        if (y<0) y+=4294967296.0;
        return(y*2097152.0+z)*(1.0/9007199254740992.0);
    }

    /* ０か１を返す乱数 */
    /*synchronized*/ public int NextBit() {
        if (--coin_bits==-1)
        { coin_bits=31; return(coin_save=NextMt())&1; }
        else return(coin_save>>>=1)&1;
    }

    /* ０から２５５を返す乱数 */
    /*synchronized*/ public int NextByte() {
        if (--byte_pos==-1)
        { byte_pos=3; return(int)(byte_save=NextMt())&255; }
        else return(int)(byte_save>>>=8)&255;
    }

    /* 丸め誤差のない０以上 range_ 未満の整数乱数 */
    /*synchronized*/ public int NextIntEx(int range_) {
        int y_,base_,remain_; int shift_;

        if (range_<=0) return 0;
        if (range_!=range) {
            base=(range=range_);
            for (shift=0;base<=(1<<30)&&base!=1<<31;shift++) base<<=1;
        }
        for (;;) {
            y_=NextMt()>>>1;
            if (y_<base||base==1<<31) return(int)(y_>>>shift);
            base_=base; shift_=shift; y_-=base_;
            remain_=(1<<31)-base_;
            for (;remain_>=(int)range_;remain_-=base_) {
                for (;base_>remain_;base_>>>=1) shift_--;
                if (y_<base_) return(int)(y_>>>shift_);
                else y_-=base_;
            }
        }
    }

}
