/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ciphermemo;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.CRC32;

/**
 * 同期用オブジェクト
 */
class TrueRandom_LockObject {
    private static TrueRandom_LockObject instance = new TrueRandom_LockObject();
    public static TrueRandom_LockObject getInstance() {
        return instance;
    }
}

/**
 * 真の乱数を生成する
 * @author Kanemitsu
 */
public class TrueRandom {
    private static CRC32 crc32;
    private static long counter = 0;

    private static int entropy_length = 1024 * 100;
    private static byte[] entropy_data = new byte[entropy_length];
    private static int entropy_WritePosition = 0;
    private static int entropy_ReadPosition = 0;

    /**
     * エントロピーの上限を設定する<br>
     * 以前のデータは失われる
     * @param len 設定するサイズ
     */
    public static void setEntropyLength(int len) {
        synchronized (TrueRandom_LockObject.getInstance()) {
            entropy_data = null;
            entropy_data = new byte[len];
            entropy_length = len;
            entropy_WritePosition = 0;
            entropy_ReadPosition = 0;
        }
    }

    public static int getEntropyLength() {
        return entropy_length;
    }
    
    public static void addEntropyData(byte data) {
        if (data == 0) return;
        synchronized (TrueRandom_LockObject.getInstance()) {
            entropy_data[entropy_WritePosition] = data;
            entropy_WritePosition++;
            if (entropy_WritePosition >= entropy_length) entropy_WritePosition = 0;
        }
    }

    public static void addEntropyData(int data) {
        synchronized (TrueRandom_LockObject.getInstance()) {
            for (int i = 0; i < 4; ++i) {
                addEntropyData((byte)(data & 0xFF));
                data >>>= 8;
            }
        }
    }

    public static void addEntropyData(long data) {
        synchronized (TrueRandom_LockObject.getInstance()) {
            for (int i = 0; i < 8; ++i) {
                addEntropyData((byte)(data & 0xFF));
                data >>>= 8;
            }
        }
    }
    
    public static byte[] convertToByteArray(int data){
        byte[] result = new byte[4];
        convertToByteArray(data, result, 0);
        return result;
    }

    public static void convertToByteArray(int data, byte[] dest, int startIndex){
        dest[startIndex + 0] = (byte)((data & 0xFF000000) >>> 24);
        dest[startIndex + 1] = (byte)((data & 0x00FF0000) >>> 16);
        dest[startIndex + 2] = (byte)((data & 0x0000FF00) >>> 8);
        dest[startIndex + 3] = (byte)(data & 0x000000FF);
    }

    public static int convertToInt(byte[] data, int startIndex) {
        return ((data[startIndex + 0] & 0xFF) << 24) | 
               ((data[startIndex + 1] & 0xFF) << 16) | 
               ((data[startIndex + 2] & 0xFF) << 8) | 
               (data[startIndex + 3] & 0xFF);
    }

    synchronized public static void getValueArray(byte[] dest) {
        for (int i = 0, n = dest.length; i < n; ++i) {
            dest[i] = (byte)getValue();
        }
    }

    public static int getValue() {
        synchronized (TrueRandom_LockObject.getInstance()) {
            if (crc32 == null) crc32 = new CRC32();

            byte[] temp = new byte[28];

            convertToByteArray((int)System.currentTimeMillis(), temp, 0);
            convertToByteArray((int)(System.currentTimeMillis() >>> 32), temp, 4);
            convertToByteArray((int)System.nanoTime(), temp, 8);
            convertToByteArray((int)(System.nanoTime() >>> 32), temp, 12);
            convertToByteArray((int)counter, temp, 16);
            convertToByteArray((int)(counter >>> 32), temp, 20);

            for (int i = 24; i < 28; ++i) {
                temp[i] = entropy_data[entropy_ReadPosition];
                entropy_ReadPosition++;
                if (entropy_ReadPosition >= entropy_length) entropy_ReadPosition = 0;
            }

            crc32.update(temp);
            long result = crc32.getValue();

            //sleepの精度が環境によって違うことを利用
            if ((counter & 0x3F) == 0) {
                try {
                    Thread.sleep(1);
                } catch (InterruptedException ex) {
                    Logger.getLogger(TrueRandom.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            counter--;
            return (int)(result & 0xFFFFFFFF);
        }
    }
}
